
import pytest
import jwt


@pytest.fixture
def token():
    def generate(overwrites=None):
        if overwrites is None:
            overwrites = {}
        data = {**{
            'claims': {
                'roles': [],
                'claims': {
                    'shops': [],
                    'organizations': []
                }
            },
            'iss': 'issuer',
            'aud': 'audition',
            'auth_time': 'time',
            'user_id': '1234',
            'sub': 'sub',
            'iat': 1595838390,
            'exp': 1595839390,
            'email': 'email@test.com',
            'email_verified': False,
            'firebase': {
                'identities': {
                    'email': ['email@test.com']
                },
                'sign_in_provider': 'custom'
            }
        }, **overwrites}
        return jwt.encode(data, 'secret-key')
    return generate


@pytest.fixture
def empty_token():
    return jwt.encode({}, 'secret-key')


# -*- coding: utf-8 -*-

""" Identity class

    Encapsulates the logic to handle JWT tokens
"""

import logging

from jwt.exceptions import InvalidTokenError
import jwt


class Identity:
    """ Holds the identity of the logged user
    """
    def __init__(self, token):
        self._token = token
        try:
            self._decoded = jwt.decode(self._token, verify=False)
        except InvalidTokenError:
            raise ValueError('Invalid JWT token')

    def user_id(self):
        """ Returns the user_id provided by firebase
        """
        return self._decoded.get('user_id')

    def is_system_admin(self):
        """ Returns a boolean identifying the user as a system administrator
        """
        return 'sysadmin' in self._roles()

    def is_admin(self):
        """ Returns a boolean identifying the user 
            as an organization administrator
        """
        return 'admin' in self._roles()

    def is_staff(self):
        """ Returns a boolean identifying the user 
            as a shop staff
        """
        return 'staff' in self._roles()

    def shops(self):
        """ Returns a list of shop_ids that the user has access to
        """
        return self._custom_claims().get('shop', [])

    def organizations(self):
        """ Returns a list of organization_ids that the user has access to
        """
        return self._custom_claims().get('organization', [])

    def data(self):
        """ Return the entire data from the token
        """
        return self._decoded

    def token(self):
        """ Returns the original JWT token
        """
        return self._token

    def _roles(self):
        """ Returns the collection of roles
        """
        if 'claims' not in self._decoded or 'roles' not in self._decoded['claims']:
            logging.error('roles not found')
            return []
        return self._decoded['claims']['roles']

    def _custom_claims(self):
        """ Returns the collection of custom claims
        """
        if 'claims' not in self._decoded or 'claims' not in self._decoded['claims']:
            logging.error('claims not found')
            return {}
        return self._decoded['claims']['claims']



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `styler_identity` package."""

from unittest.mock import patch
import pytest


from styler_identity.identity import Identity


class TestIdentity:
    """ Tests 
    """
    def test_invalid_token(self):
        with pytest.raises(ValueError) as expected:
            Identity('invalid token')

        assert str(expected.value) == 'Invalid JWT token'

    def test_valid_token(self, token):
        idem = Identity(token())

        assert isinstance(idem, Identity)

    def test_token(self, token):
        original_token = token()
        idem = Identity(original_token)

        tk = idem.token()

        assert original_token == tk

    def test_user_id(self, token):
        idem = Identity(token({'user_id': '1234'}))

        user_id = idem.user_id()

        assert user_id == '1234'

    def test_is_system_admin(self, token):
        idem = Identity(token({
            'claims': {
                'roles':['sysadmin']
            }
        }))
        idem2 = Identity(token({
            'claims': {
                'roles':['admin']
            }
        }))

        it_should_be = idem.is_system_admin()
        is_should_not_be = idem2.is_system_admin()

        assert it_should_be
        assert not is_should_not_be

    def test_is_admin(self, token):
        idem = Identity(token({
            'claims': {
                'roles':['admin']
            }
        }))
        idem2 = Identity(token({
            'claims': {
                'roles':['staff']
            }
        }))

        it_should_be = idem.is_admin()
        is_should_not_be = idem2.is_admin()

        assert it_should_be
        assert not is_should_not_be

    def test_is_staff(self, token):
        idem = Identity(token({
            'claims': {
                'roles':['staff']
            }
        }))
        idem2 = Identity(token({
            'claims': {
                'roles':['admin']
            }
        }))

        it_should_be = idem.is_staff()
        is_should_not_be = idem2.is_staff()

        assert it_should_be
        assert not is_should_not_be

    @patch('logging.error')
    def test_invalid_roles_sysadmin(self, mocked_error, empty_token):
        idem = Identity(empty_token)

        result = idem.is_system_admin()

        assert not result
        mocked_error.assert_called_once

    @patch('logging.error')
    def test_invalid_roles_admin(self, mocked_error, empty_token):
        idem = Identity(empty_token)

        result = idem.is_admin()

        assert not result
        mocked_error.assert_called_once

    @patch('logging.error')
    def test_invalid_roles_staff(self, mocked_error, empty_token):
        idem = Identity(empty_token)

        result = idem.is_staff()

        assert not result
        mocked_error.assert_called_once

    def test_shops(self, token):
        idem = Identity(token({
            'claims': {
                'claims': {
                    'shop': ['12345', '33442']
                }
            }
        }))

        shops = idem.shops()

        assert shops == ['12345', '33442']

    @patch('logging.error')
    def test_shops_none(self, mocked_error, empty_token):
        idem = Identity(empty_token)

        shops = idem.shops()

        assert shops == []
        mocked_error.assert_called_once

    def test_organizations(self, token):
        idem = Identity(token({
            'claims': {
                'claims': {
                    'organization': ['33333']
                }
            }
        }))

        organizations = idem.organizations()

        assert organizations == ['33333']

    @patch('logging.error')
    def test_organizations_none(self, mocked_error, empty_token):
        idem = Identity(empty_token)

        organizations = idem.organizations()

        assert organizations == []
        mocked_error.assert_called_once

    def test_data(self, token):
        idem = Identity(token())
        expected_keys = {
            'claims',
            'iss',
            'aud',
            'auth_time',
            'user_id',
            'sub',
            'iat',
            'exp',
            'email',
            'email_verified',
            'firebase'
        }

        data = idem.data()

        assert set(data.keys()) == expected_keys



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
===============================
HtmlTestRunner
===============================


.. image:: https://img.shields.io/pypi/v/styler_identity.svg
        :target: https://pypi.python.org/pypi/styler_identity

Simple library used to handle user data from JWT tokens


Links:
---------
* `Github <https://github.com/STYLER-Inc/styler-identity>`_
"""

from setuptools import setup, find_packages

requirements = ['pyjwt']

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest', ]

setup(
    author="Bruno Toshio Sugano",
    author_email='bruno.sugano@styler.link',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
    ],
    description="Simple library used to handle user data from JWT tokens",
    install_requires=requirements,
    license="MIT license",
    long_description=__doc__,
    include_package_data=True,
    keywords='styler_identity',
    name='styler_identity',
    packages=find_packages(include=['styler_identity']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/STYLER-Inc/styler-identity',
    version='0.5.1',
    zip_safe=False,
)



# -*- coding: utf-8 -*-

"""Top-level package for Styler Identity."""

__author__ = """Bruno Toshio Sugano"""
__email__ = 'bruno.sugano@styler.link'
__version__ = '0.5.1'


from styler_identity.identity import Identity
